<?php

namespace App\Http\Controllers;

use App\Mail\ChosenHomeMail;
use App\Models\Attraction;
use App\Models\Homes;
use App\Models\HomesCategory;
use App\Models\HomesAvailability;
use App\Models\Locale;
use App\Rules\ReCAPTCHAv3;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Session;

class ChosenHomeController extends Controller
{
    public function show($Code){
        $data[ 'attraction' ] = Attraction::where('Locale', '=', App::getlocale())->get();
        $data[ 'home' ] = Homes::where('Locale', App::getlocale())->where('Code', $Code)->get();
        $data[ 'homes_category' ] = HomesCategory::where('Locale', '=', App::getlocale())->get();

        return view('website.chosenHome', compact( 'data'));
    }

    public function send (Request $request){
        $Id = $this->loadData($request);
        if($Id != false) {
            $this->sendMail($Id);
            return response()->json([ 'alert' => 'success', 'message' => 'Successful!' ]);
        }
        else response()->json([ 'alert' => 'error', 'message' => 'Error!' ]);
    }

    public function loadData ($request){
        $HomesAvailability = new HomesAvailability;

        $request->validate([
            'Home' => 'required',
            'CheckIn' => 'required',
            'CheckOut' => 'required',
            'Guest' => 'required',
            'Name' => 'required',
            'Email' => 'required',
            'Phone' => 'required',
            'token' => ['required', new ReCAPTCHAv3]
        ]);

        $HomesAvailability->Home = $request->input('Home');
        $HomesAvailability->CheckIn = Carbon::createFromFormat('d/m/Y', $request->input('CheckIn'))->format('Y-m-d 00:00:00');
        $HomesAvailability->CheckOut = Carbon::createFromFormat('d/m/Y', $request->input('CheckOut'))->format('Y-m-d 00:00:00');
        $HomesAvailability->Guest = $request->input('Guest');
        $HomesAvailability->Name = $request->input('Name');
        $HomesAvailability->Email = $request->input('Email');
        $HomesAvailability->Phone = $request->input('Phone');

        if($HomesAvailability->save()) return $HomesAvailability->id;
        else return false;
    }

    public function sendMail ($Id){
        $HomesAvailability = HomesAvailability::findOrFail($Id);
        $Home = $HomesAvailability->Home()->get()[0];
        $HomeCategory = $Home->HomeCategory()->get()[0];

        Mail::to(Config::get('mail.from.address'))->send(new ChosenHomeMail($HomesAvailability, $Home, $HomeCategory));
    }

}
